import React, { ChangeEvent, Component, MouseEvent } from 'react';

type Language = "en" | "es" | "jp";

type HelloAppProps = {};
type HelloAppState = {
  showGreeting: boolean,
  name: string,
  language: Language,
};

const languageToGreeting = (lang: Language): string => {
  if (lang === "jp") {
    return "こんにちは";
  } else if (lang === "es") {
    return "Hola";
  } else {
    return "Hi";
  }
}

export class HelloApp extends Component<HelloAppProps, HelloAppState> {

  constructor(props: HelloAppProps) {
    super(props);

    this.state = {showGreeting: false, name: "", language: "en"};
  }

  render = (): JSX.Element => {
    if (this.state.showGreeting) {
      const greeting = languageToGreeting(this.state.language);
      return <>
        <p lang={this.state.language}>{greeting}, {this.state.name}!</p>
        {this.renderLanguageSelect()}
      </>;
    } else {
      return <>
        <p>What is your name?{" "}
          <input type="text" value={this.state.name} onChange={this.doNameChange} />
          <button onClick={this.doDoneClick}>Done</button>
        </p>
        {this.renderLanguageSelect()}
      </>;
    }
  };

  renderLanguageSelect = (): JSX.Element => {
    return <select onChange={this.doLanguageChange} value={this.state.language}>
      <option value="en">English</option>
      <option value="es">Spanish</option>
      <option value="jp">Japanese</option>
    </select>
  }

  doLanguageChange = (evt: ChangeEvent<HTMLSelectElement>): void => {
    const newLanguage = evt.target.value;

    if (!(newLanguage === "en" || newLanguage === "es" || newLanguage === "jp")) {
      throw new Error(`Invalid language: ${newLanguage}`);
    }

    this.setState({language: newLanguage})
  }

  doNameChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({name: evt.target.value});
  };

  doDoneClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    this.setState({showGreeting: true});
  };
}
